﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace _2011_06_07_2011_06_08_Praktkum
{
    abstract class TreemapComponent
    {
        long numberOfBytes;

        public long NumberOfBytes
        {
            get
            {
                return numberOfBytes;
            }
            protected set
            {
                numberOfBytes = value;
            }
        }

        protected System.Windows.Rect rect;
        public abstract void AddToCanvas(System.Windows.Controls.Canvas canvas, System.Windows.Rect rect, bool vertical);

        string name;

        public string Name
        {
            get
            {
                return name;
            }
            protected set
            {
                name = value;
            }
        }

        public abstract TreemapComponent FindOnScreen(System.Windows.Point point);

        protected static Random random = new Random();

        protected const double padding = 5.0;
    }

    class TreemapLeaf : TreemapComponent
    {
        public TreemapLeaf(System.IO.FileInfo file)
        {
            NumberOfBytes = file.Length;
            Name = file.Name;
        }

        public override void AddToCanvas(System.Windows.Controls.Canvas canvas, System.Windows.Rect rect, bool vertical)
        {
            this.rect = rect;
            System.Windows.Shapes.Rectangle r = new System.Windows.Shapes.Rectangle();
            r.Width = rect.Width;
            r.Height = rect.Height;
            System.Windows.Media.Color c = new System.Windows.Media.Color();
            c.R = (byte)(128 + random.Next(128)); // eher helle Farben
            c.G = (byte)(128 + random.Next(128));
            c.B = (byte)(128 + random.Next(128));
            c.A = (byte)255;
            r.Fill = new System.Windows.Media.SolidColorBrush(c);
            System.Windows.Controls.Canvas.SetLeft(r, rect.X);
            System.Windows.Controls.Canvas.SetTop(r, rect.Y);
            canvas.Children.Add(r);
        }

        public override TreemapComponent FindOnScreen(System.Windows.Point point)
        {
            if (rect.Contains(point))
            {
                return this;
            }
            else
            {
                return null;
            }
        }
    }

    class TreemapComposite : TreemapComponent
    {
        List<TreemapComponent> components = new List<TreemapComponent>();

        public TreemapComposite(System.IO.DirectoryInfo directory, int levels)
        {
            Name = directory.Name;

            foreach (System.IO.FileInfo file in directory.GetFiles())
            {
                TreemapLeaf f = new TreemapLeaf(file);
                components.Add(f);
            }

            if (levels > 0)
            {
                foreach (System.IO.DirectoryInfo dir in directory.GetDirectories())
                {
                    TreemapComposite d = new TreemapComposite(dir, levels - 1);
                    components.Add(d);
                }
            }

            NumberOfBytes = components.Sum(x => x.NumberOfBytes);
            components.Sort((x,y) => -x.NumberOfBytes.CompareTo(y.NumberOfBytes));
        }

        public override void AddToCanvas(System.Windows.Controls.Canvas canvas, System.Windows.Rect rect, bool vertical)
        {
            this.rect = rect;

            // das Innere zeichnen
            double z = 0.0;
            foreach (TreemapComponent comp in components)
            {
                double zNext = z + comp.NumberOfBytes / (double)this.NumberOfBytes;
                double x, y, w, h;

                if (vertical)
                {
                    x = rect.X + padding + (rect.Width - 2.0 * padding) * z;
                    w = (rect.Width - 2.0 * padding) * (zNext - z);
                    y = rect.Y + padding;
                    h = rect.Height - 2.0 * padding;
                }
                else
                {
                    y = rect.Y + padding + (rect.Height - 2.0 * padding) * z;
                    h = (rect.Height - 2.0 * padding) * (zNext - z);
                    x = rect.X + padding;
                    w = rect.Width - 2.0 * padding;
                }

                if (h > 0.0 && w > 0.0) // ist nicht zu klein
                {
                    System.Windows.Rect rc = new System.Windows.Rect(x, y, w, h);
                    comp.AddToCanvas(canvas, rc, !vertical);
                }

                z = zNext;
            }

            // den Rahmen zeichnen
            System.Windows.Shapes.Rectangle r = new System.Windows.Shapes.Rectangle();
            r.Width = rect.Width;
            r.Height = rect.Height;
            r.Stroke = System.Windows.Media.Brushes.Black;
            r.StrokeThickness = 1.0;
            System.Windows.Controls.Canvas.SetLeft(r, rect.X);
            System.Windows.Controls.Canvas.SetTop(r, rect.Y);
            canvas.Children.Add(r);
        }

        public override TreemapComponent FindOnScreen(System.Windows.Point point)
        {
            foreach (TreemapComponent c in components)
            {
                TreemapComponent d = c.FindOnScreen(point);
                if (d != null)
                {
                    return d;
                }
            }
            return null;
        }
    }
}
